module Refltr

class Test

  #
  # Create a test generator object
  #

	def initialize(name)
		if !self.respond_to?("test_#{name}", true)
			raise "Undefined test #{name}"
		end

		@name = name
	end

  #
  # Get the data generated by the test function
  #

	def data
		self.send("test_#{@name}")
	end

  #
  # Get a list of all tests
  #

	def self.list
		tests = []

		self.private_instance_methods.each { |name|
			match = /^test_(.*)/.match(name)
			if match
				tests.push(match[1])
			end
		}

		tests
	end

private

	HTML_ELEMENTS = ['a','abbr','acronym','address','applet','area','b','base','baseFont','bdo','bgSound','big','blockQuote','br','button','caption','center','cite','code','col','colGroup','comment','custom','dd','del','dfn','dir','div','dl','dt','em','embed','fieldSet','font','form','frame','frameSet','head','h1','h2','h3','h4','h5','h6','hr','html','i','iframe','img','input','ins','isIndex','kbd','label','legend','li','link','listing','map','marquee','menu','meta','noBR','noFrames','noScript','object','ol','optGroup','option','p','param','pre','q','rt','ruby','s','samp','script','select','small','span','strike','strong','style','sub','sup','table','tBody','td','textArea','tFoot','th','tHead','title','tr','tt','u','ul','var','wbr','xml','xmp','blink','canvas','certificate','ilayer','keygen','layer','nextid','noembed','nolayer','sound','spacer']

	def test_html_text_null
		"a\x00b"
	end

	def test_html_text_lineendings
		"a\x0ab\x0dc\x0a\x0dd\x0d\x0ae"
	end

	def test_html_text_ascii
		data = ''
		(0x01..0xff).each { |i|
			data << i.chr
		}
		return data
	end

	def test_html_text_entities
		data = ''

		(0x00..0xff).each { |i|
			data << sprintf("&#%d\n", i)
			data << sprintf("&#%d;\n", i)
			data << sprintf("&#%02d;\n", i)
			data << sprintf("&#%07d;\n", i)
			data << sprintf("&#x%x\n", i)
			data << sprintf("&#x%x;\n", i)
			data << sprintf("&#x%02x;\n", i)
			data << sprintf("&#x%07x;\n", i)
		}

		["&quot\n", "&quot;\n", "&amp\n", "&amp;\n", "&apos\n", "&apos;\n",
		"&lt\n", "&lt;\n", "&gt\n", "&gt;\n", "&nbsp\n", "&nbsp;\n", "&iexcl\n",
		"&iexcl;\n", "&cent\n", "&cent;\n", "&pound\n", "&pound;\n", "&curren\n",
		"&curren;\n", "&yen\n", "&yen;\n", "&brvbar\n", "&brvbar;\n", "&sect\n",
		"&sect;\n", "&uml\n", "&uml;\n", "&copy\n", "&copy;\n", "&ordf\n",
		"&ordf;\n", "&laquo\n", "&laquo;\n", "&not\n", "&not;\n", "&shy\n",
		"&shy;\n", "&reg\n", "&reg;\n", "&macr\n", "&macr;\n", "&deg\n", "&deg;\n",
		"&plusmn\n", "&plusmn;\n", "&sup2\n", "&sup2;\n", "&sup3\n", "&sup3;\n",
		"&acute\n", "&acute;\n", "&micro\n", "&micro;\n", "&para\n", "&para;\n",
		"&middot\n", "&middot;\n", "&cedil\n", "&cedil;\n", "&sup1\n", "&sup1;\n",
		"&ordm\n", "&ordm;\n", "&raquo\n", "&raquo;\n", "&frac14\n", "&frac14;\n",
		"&frac12\n", "&frac12;\n", "&frac34\n", "&frac34;\n", "&iquest\n",
		"&iquest;\n", "&Agrave\n", "&Agrave;\n", "&Aacute\n", "&Aacute;\n",
		"&Acirc\n", "&Acirc;\n", "&Atilde\n", "&Atilde;\n", "&Auml\n", "&Auml;\n",
		"&Aring\n", "&Aring;\n", "&AElig\n", "&AElig;\n", "&Ccedil\n",
		"&Ccedil;\n", "&Egrave\n", "&Egrave;\n", "&Eacute\n", "&Eacute;\n",
		"&Ecirc\n", "&Ecirc;\n", "&Euml\n", "&Euml;\n", "&Igrave\n", "&Igrave;\n",
		"&Iacute\n", "&Iacute;\n", "&Icirc\n", "&Icirc;\n", "&Iuml\n", "&Iuml;\n",
		"&ETH\n", "&ETH;\n", "&Ntilde\n", "&Ntilde;\n", "&Ograve\n", "&Ograve;\n",
		"&Oacute\n", "&Oacute;\n", "&Ocirc\n", "&Ocirc;\n", "&Otilde\n",
		"&Otilde;\n", "&Ouml\n", "&Ouml;\n", "&times\n", "&times;\n", "&Oslash\n",
		"&Oslash;\n", "&Ugrave\n", "&Ugrave;\n", "&Uacute\n", "&Uacute;\n",
		"&Ucirc\n", "&Ucirc;\n", "&Uuml\n", "&Uuml;\n", "&Yacute\n", "&Yacute;\n",
		"&THORN\n", "&THORN;\n", "&szlig\n", "&szlig;\n", "&agrave\n",
		"&agrave;\n", "&aacute\n", "&aacute;\n", "&acirc\n", "&acirc;\n",
		"&atilde\n", "&atilde;\n", "&auml\n", "&auml;\n", "&aring\n", "&aring;\n",
		"&aelig\n", "&aelig;\n", "&ccedil\n", "&ccedil;\n", "&egrave\n",
		"&egrave;\n", "&eacute\n", "&eacute;\n", "&ecirc\n", "&ecirc;\n",
		"&euml\n", "&euml;\n", "&igrave\n", "&igrave;\n", "&iacute\n",
		"&iacute;\n", "&icirc\n", "&icirc;\n", "&iuml\n", "&iuml;\n", "&eth\n",
		"&eth;\n", "&ntilde\n", "&ntilde;\n", "&ograve\n", "&ograve;\n",
		"&oacute\n", "&oacute;\n", "&ocirc\n", "&ocirc;\n", "&otilde\n",
		"&otilde;\n", "&ouml\n", "&ouml;\n", "&divide\n", "&divide;\n",
		"&oslash\n", "&oslash;\n", "&ugrave\n", "&ugrave;\n", "&uacute\n",
		"&uacute;\n", "&ucirc\n", "&ucirc;\n", "&uuml\n", "&uuml;\n", "&yacute\n",
		"&yacute;\n", "&thorn\n", "&thorn;\n", "&yuml\n", "&yuml;\n", "&OElig\n",
		"&OElig;\n", "&oelig\n", "&oelig;\n", "&Scaron\n", "&Scaron;\n",
		"&scaron\n", "&scaron;\n", "&Yuml\n", "&Yuml;\n", "&fnof\n", "&fnof;\n",
		"&circ\n", "&circ;\n", "&tilde\n", "&tilde;\n", "&Alpha\n", "&Alpha;\n",
		"&Beta\n", "&Beta;\n", "&Gamma\n", "&Gamma;\n", "&Delta\n", "&Delta;\n",
		"&Epsilon\n", "&Epsilon;\n", "&Zeta\n", "&Zeta;\n", "&Eta\n", "&Eta;\n",
		"&Theta\n", "&Theta;\n", "&Iota\n", "&Iota;\n", "&Kappa\n", "&Kappa;\n",
		"&Lambda\n", "&Lambda;\n", "&Mu\n", "&Mu;\n", "&Nu\n", "&Nu;\n", "&Xi\n",
		"&Xi;\n", "&Omicron\n", "&Omicron;\n", "&Pi\n", "&Pi;\n", "&Rho\n",
		"&Rho;\n", "&Sigma\n", "&Sigma;\n", "&Tau\n", "&Tau;\n", "&Upsilon\n",
		"&Upsilon;\n", "&Phi\n", "&Phi;\n", "&Chi\n", "&Chi;\n", "&Psi\n",
		"&Psi;\n", "&Omega\n", "&Omega;\n", "&alpha\n", "&alpha;\n", "&beta\n",
		"&beta;\n", "&gamma\n", "&gamma;\n", "&delta\n", "&delta;\n", "&epsilon\n",
		"&epsilon;\n", "&zeta\n", "&zeta;\n", "&eta\n", "&eta;\n", "&theta\n",
		"&theta;\n", "&iota\n", "&iota;\n", "&kappa\n", "&kappa;\n", "&lambda\n",
		"&lambda;\n", "&mu\n", "&mu;\n", "&nu\n", "&nu;\n", "&xi\n", "&xi;\n",
		"&omicron\n", "&omicron;\n", "&pi\n", "&pi;\n", "&rho\n", "&rho;\n",
		"&sigmaf\n", "&sigmaf;\n", "&sigma\n", "&sigma;\n", "&tau\n", "&tau;\n",
		"&upsilon\n", "&upsilon;\n", "&phi\n", "&phi;\n", "&chi\n", "&chi;\n",
		"&psi\n", "&psi;\n", "&omega\n", "&omega;\n", "&thetasym\n",
		"&thetasym;\n", "&upsih\n", "&upsih;\n", "&piv\n", "&piv;\n", "&ensp\n",
		"&ensp;\n", "&emsp\n", "&emsp;\n", "&thinsp\n", "&thinsp;\n", "&zwnj\n",
		"&zwnj;\n", "&zwj\n", "&zwj;\n", "&lrm\n", "&lrm;\n", "&rlm\n", "&rlm;\n",
		"&ndash\n", "&ndash;\n", "&mdash\n", "&mdash;\n", "&lsquo\n", "&lsquo;\n",
		"&rsquo\n", "&rsquo;\n", "&sbquo\n", "&sbquo;\n", "&ldquo\n", "&ldquo;\n",
		"&rdquo\n", "&rdquo;\n", "&bdquo\n", "&bdquo;\n", "&dagger\n",
		"&dagger;\n", "&Dagger\n", "&Dagger;\n", "&bull\n", "&bull;\n",
		"&hellip\n", "&hellip;\n", "&permil\n", "&permil;\n", "&prime\n",
		"&prime;\n", "&Prime\n", "&Prime;\n", "&lsaquo\n", "&lsaquo;\n",
		"&rsaquo\n", "&rsaquo;\n", "&oline\n", "&oline;\n", "&frasl\n",
		"&frasl;\n", "&euro\n", "&euro;\n", "&image\n", "&image;\n", "&weierp\n",
		"&weierp;\n", "&real\n", "&real;\n", "&trade\n", "&trade;\n", "&alefsym\n",
		"&alefsym;\n", "&larr\n", "&larr;\n", "&uarr\n", "&uarr;\n", "&rarr\n",
		"&rarr;\n", "&darr\n", "&darr;\n", "&harr\n", "&harr;\n", "&crarr\n",
		"&crarr;\n", "&lArr\n", "&lArr;\n", "&uArr\n", "&uArr;\n", "&rArr\n",
		"&rArr;\n", "&dArr\n", "&dArr;\n", "&hArr\n", "&hArr;\n", "&forall\n",
		"&forall;\n", "&part\n", "&part;\n", "&exist\n", "&exist;\n", "&empty\n",
		"&empty;\n", "&nabla\n", "&nabla;\n", "&isin\n", "&isin;\n", "&notin\n",
		"&notin;\n", "&ni\n", "&ni;\n", "&prod\n", "&prod;\n", "&sum\n", "&sum;\n",
		"&minus\n", "&minus;\n", "&lowast\n", "&lowast;\n", "&radic\n",
		"&radic;\n", "&prop\n", "&prop;\n", "&infin\n", "&infin;\n", "&ang\n",
		"&ang;\n", "&and\n", "&and;\n", "&or\n", "&or;\n", "&cap\n", "&cap;\n",
		"&cup\n", "&cup;\n", "&int\n", "&int;\n", "&there4\n", "&there4;\n",
		"&sim\n", "&sim;\n", "&cong\n", "&cong;\n", "&asymp\n", "&asymp;\n",
		"&ne\n", "&ne;\n", "&equiv\n", "&equiv;\n", "&le\n", "&le;\n", "&ge\n",
		"&ge;\n", "&sub\n", "&sub;\n", "&sup\n", "&sup;\n", "&nsub\n", "&nsub;\n",
		"&sube\n", "&sube;\n", "&supe\n", "&supe;\n", "&oplus\n", "&oplus;\n",
		"&otimes\n", "&otimes;\n", "&perp\n", "&perp;\n", "&sdot\n", "&sdot;\n",
		"&lceil\n", "&lceil;\n", "&rceil\n", "&rceil;\n", "&lfloor\n",
		"&lfloor;\n", "&rfloor\n", "&rfloor;\n", "&lang\n", "&lang;\n", "&rang\n",
		"&rang;\n", "&loz\n", "&loz;\n", "&spades\n", "&spades;\n", "&clubs\n",
		"&clubs;\n", "&hearts\n", "&hearts;\n", "&diams\n", "&diams;\n"].each { |e|
			data << e
		}

		data << "&foo;\n"

		return data
	end

	def test_html_text_utf8
		"a\xd1\x8cb\xc0\xafc"
	end

	def test_html_cdata
		data = "a<![CDATA[ foo ]]>b\n"+
			"a<!x x x>b\n"

		(1..0xFF).each { |b|
			data << "<!#{b.chr}>#{sprintf('0x%02x', b)}\n"
		}

		data << "<b>b</b>\n"
	end

	def test_html_comment
		"a<!-- foo -->b\n"+
		"a<!-- foo -- >b\n"+
		"a<!- foo>b\n"
	end

	def test_html_pi
		data = "a<?x x x>b\n"

		(1..0xFF).each { |b|
			data << "<?#{b.chr}>#{sprintf('0x%02x', b)}\n"
		}

		data << "<b>b</b>\n"
	end

	def test_html_tag_nesting
		"a</b>\n"+
		"a<b>b</b>c\n"+
		"a<b>b<b>c</b>d</b>e\n"+
		"a<b>b<i>c</i>d</b>e\n"+
		"a<b>b<i>c</b>d</i>e\n"+
		"a<b>b</i>c</b>d\n" +
		"<p>p1<p>p2</p></p>\n" +
		"a<b>c\n"
	end

	def test_html_tag_nesting_level_b
		data = ''
		(1..200).each { |i|
			data << "<b>#{i}\n"
		}
		data << "<div>div</div>\n"
		data
	end

	def test_html_tag_nesting_level_div
		data = ''
		(1..200).each { |i|
			data << "<div>#{i}\n"
		}
		data << "</div></div><div><div><div>\n"
		data << "<b>b</b>\n"
		data
	end

	def test_html_tag_nesting_level_mixed
		data = ''
		(1..200).each { |i|
			data << "<b>#{i}\n"
		}
		(201..250).each { |i|
			data << "<div>#{i}\n"
		}
		data
	end

	def test_html_tag_open
		data = ''
		(1..0xFF).each { |b|
			data << "<#{b.chr}>a</#{b.chr}> #{sprintf('0x%02x', b)}\n"
		}

		(1..0xFF).each { |b|
			data << "<b#{b.chr}>a</b#{b.chr}> b #{sprintf('0x%02x', b)}\n"
		}

		(1..0xFF).each { |b|
			data << "<#{b.chr}b>a</#{b.chr}b> #{sprintf('0x%02x', b)} b\n"
			if b.chr == '<'
				data << "</b>\n";
			end
		}

		data
	end

	def test_html_tag_unknown
		data = "<x>x</x>a\n" +
			"<b><x>x</x>a</b>\n" +
			"<b><i><x>x</x>a</i></b>\n" +
			"<b><x>x</x>a<i>b</i></b>\n" +
			"<b><x>x</x>a<a>b</a></b>\n" +
			"<b><x>x</x>a<br></b>\n"
	end

	def test_html_tag_closing
		data = "p </p> p\n" +
			"div </div> div\n" +
			"b </b> b\n" +
			"z </z> z\n"

		(1..0xFF).each { |b|
			data << "</#{b.chr}> #{sprintf('0x%02x', b)}\n"
		}

		(1..0xFF).each { |b|
			data << "<#{b.chr}>a</>b</#{b.chr}> #{sprintf('0x%02x', b)}\n"
		}

		(1..0xFF).each { |b|
			data << "<z>a</#{b.chr}>z</z> #{sprintf('0x%02x', b)}\n"
		}
		data
	end
	
	def test_html_element_whitelist
		data = ''

		HTML_ELEMENTS.each { |e|
			data << "<#{e}>a</#{e}> #{e}\n"
		}

		['button','checkbox','file','hidden','image','password','radio','reset','submit','test'].each { |t|
			data << "<input type=\"#{t}\">#{t}</input> #{t}\n"
		}

		data
	end

	def test_html_element_plaintext
		"a<plaintext>b</plaintext>c"
	end

	def test_html_element_body
		"a<body>b</body>c"
	end

	def test_html_attr_parsing
		data = ''

		(1..0xFF).each { |b|
			data << "<b#{b.chr}a b>a</></a></b></b> #{sprintf('0x%02x', b)}\n"
		}

		(1..0xFF).each { |b|
			data << "<b #{b.chr}a b>a</></a></b></b> #{sprintf('0x%02x', b)}\n"
		}

		(1..0xFF).each { |b|
			data << "<b a#{b.chr}b>a</></a></b></b> #{sprintf('0x%02x', b)}\n"
		}

		data
	end

	def test_html_attr_closing
		data = "<b/a>b</b>\n" +
 			"<b//a>b</b>\n" +
 			"<b /a>b</b>\n" +
 			"<b //a>b</b>\n" +
			"<b a/b>b</b>\n" +
			"<b a//b>b</b>\n" +
			"<b a b/>b</b>\n" +
			"<b a b/c>b</b>\n"
	end

	def test_html_attr_value_parsing
		data = ''

		(1..0xFF).each { |b|
			data << "<b a#{b.chr}b#{b.chr}>a</></b></b> #{sprintf('0x%02x', b)}\n"
		}

		(1..0xFF).each { |b|
			data << "<b a#{b.chr}=b>a</></b></b> #{sprintf('0x%02x', b)}\n"
		}

		(1..0xFF).each { |b|
			data << "<b a #{b.chr}b>a</></b></b> #{sprintf('0x%02x', b)}\n"
		}

		(1..0xFF).each { |b|
			data << "<b a=#{b.chr}b#{b.chr}>a</></b></b> #{sprintf('0x%02x', b)}\n"
		}

		(1..0xFF).each { |b|
			data << "<b a=\"b#{b.chr}\">a</></b></b> #{sprintf('0x%02x', b)}\n"
		}

		data
	end

	def test_html_attr_length
		data = '<b foo=' + ("A"*10000) + '"BBB bar=bar></b>';
	end

end

end
